@@ -9,47 +9,48 @@
 #include <rapidfuzz/details/simd.hpp>
 #include <type_traits>
 
-namespace rapidfuzz::detail {
+namespace rapidfuzz {
+namespace detail {
 
 template <typename T, typename... Args>
 struct NormalizedMetricBase {
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static double normalized_distance(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
                                       Args... args, double score_cutoff, double score_hint)
     {
-        return _normalized_distance(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
-                                    score_cutoff, score_hint);
+        return _normalized_distance(make_range(first1, last1), make_range(first2, last2),
+                                    std::forward<Args>(args)..., score_cutoff, score_hint);
     }
 
     template <typename Sentence1, typename Sentence2>
     static double normalized_distance(const Sentence1& s1, const Sentence2& s2, Args... args,
                                       double score_cutoff, double score_hint)
     {
-        return _normalized_distance(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff,
+        return _normalized_distance(make_range(s1), make_range(s2), std::forward<Args>(args)..., score_cutoff,
                                     score_hint);
     }
 
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static double normalized_similarity(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2,
                                         Args... args, double score_cutoff, double score_hint)
     {
-        return _normalized_similarity(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
-                                      score_cutoff, score_hint);
+        return _normalized_similarity(make_range(first1, last1), make_range(first2, last2),
+                                      std::forward<Args>(args)..., score_cutoff, score_hint);
     }
 
     template <typename Sentence1, typename Sentence2>
     static double normalized_similarity(const Sentence1& s1, const Sentence2& s2, Args... args,
                                         double score_cutoff, double score_hint)
     {
-        return _normalized_similarity(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff,
-                                      score_hint);
+        return _normalized_similarity(make_range(s1), make_range(s2), std::forward<Args>(args)...,
+                                      score_cutoff, score_hint);
     }
 
 protected:
     template <typename InputIt1, typename InputIt2>
-    static double _normalized_distance(Range<InputIt1> s1, Range<InputIt2> s2, Args... args,
+    static double _normalized_distance(const Range<InputIt1>& s1, const Range<InputIt2>& s2, Args... args,
                                        double score_cutoff, double score_hint)
     {
         auto maximum = T::maximum(s1, s2, args...);
@@ -63,7 +64,7 @@ protected:
     }
 
     template <typename InputIt1, typename InputIt2>
-    static double _normalized_similarity(Range<InputIt1> s1, Range<InputIt2> s2, Args... args,
+    static double _normalized_similarity(const Range<InputIt1>& s1, const Range<InputIt2>& s2, Args... args,
                                          double score_cutoff, double score_hint)
     {
         double cutoff_score = NormSim_to_NormDist(score_cutoff);
@@ -74,18 +75,19 @@ protected:
         return (norm_sim >= score_cutoff) ? norm_sim : 0.0;
     }
 
-    NormalizedMetricBase(){};
+    NormalizedMetricBase()
+    {}
     friend T;
 };
 
 template <typename T, typename ResType, int64_t WorstSimilarity, int64_t WorstDistance, typename... Args>
 struct DistanceBase : public NormalizedMetricBase<T, Args...> {
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static ResType distance(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2, Args... args,
                             ResType score_cutoff, ResType score_hint)
     {
-        return T::_distance(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
+        return T::_distance(make_range(first1, last1), make_range(first2, last2), std::forward<Args>(args)...,
                             score_cutoff, score_hint);
     }
 
@@ -93,15 +95,16 @@ struct DistanceBase : public NormalizedMetricBase<T, Args...> {
     static ResType distance(const Sentence1& s1, const Sentence2& s2, Args... args, ResType score_cutoff,
                             ResType score_hint)
     {
-        return T::_distance(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff, score_hint);
+        return T::_distance(make_range(s1), make_range(s2), std::forward<Args>(args)..., score_cutoff,
+                            score_hint);
     }
 
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static ResType similarity(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2, Args... args,
                               ResType score_cutoff, ResType score_hint)
     {
-        return _similarity(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
+        return _similarity(make_range(first1, last1), make_range(first2, last2), std::forward<Args>(args)...,
                            score_cutoff, score_hint);
     }
 
@@ -109,7 +112,8 @@ struct DistanceBase : public NormalizedMetricBase<T, Args...> {
     static ResType similarity(const Sentence1& s1, const Sentence2& s2, Args... args, ResType score_cutoff,
                               ResType score_hint)
     {
-        return _similarity(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff, score_hint);
+        return _similarity(make_range(s1), make_range(s2), std::forward<Args>(args)..., score_cutoff,
+                           score_hint);
     }
 
 protected:
@@ -128,18 +132,19 @@ protected:
         return (sim >= score_cutoff) ? sim : 0;
     }
 
-    DistanceBase(){};
+    DistanceBase()
+    {}
     friend T;
 };
 
 template <typename T, typename ResType, int64_t WorstSimilarity, int64_t WorstDistance, typename... Args>
 struct SimilarityBase : public NormalizedMetricBase<T, Args...> {
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static ResType distance(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2, Args... args,
                             ResType score_cutoff, ResType score_hint)
     {
-        return _distance(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
+        return _distance(make_range(first1, last1), make_range(first2, last2), std::forward<Args>(args)...,
                          score_cutoff, score_hint);
     }
 
@@ -147,29 +152,31 @@ struct SimilarityBase : public NormalizedMetricBase<T, Args...> {
     static ResType distance(const Sentence1& s1, const Sentence2& s2, Args... args, ResType score_cutoff,
                             ResType score_hint)
     {
-        return _distance(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff, score_hint);
+        return _distance(make_range(s1), make_range(s2), std::forward<Args>(args)..., score_cutoff,
+                         score_hint);
     }
 
     template <typename InputIt1, typename InputIt2,
-              typename = std::enable_if_t<!std::is_same_v<InputIt2, double>>>
+              typename = rapidfuzz::rf_enable_if_t<!std::is_same<InputIt2, double>::value>>
     static ResType similarity(InputIt1 first1, InputIt1 last1, InputIt2 first2, InputIt2 last2, Args... args,
                               ResType score_cutoff, ResType score_hint)
     {
-        return T::_similarity(Range(first1, last1), Range(first2, last2), std::forward<Args>(args)...,
-                              score_cutoff, score_hint);
+        return T::_similarity(make_range(first1, last1), make_range(first2, last2),
+                              std::forward<Args>(args)..., score_cutoff, score_hint);
     }
 
     template <typename Sentence1, typename Sentence2>
     static ResType similarity(const Sentence1& s1, const Sentence2& s2, Args... args, ResType score_cutoff,
                               ResType score_hint)
     {
-        return T::_similarity(Range(s1), Range(s2), std::forward<Args>(args)..., score_cutoff, score_hint);
+        return T::_similarity(make_range(s1), make_range(s2), std::forward<Args>(args)..., score_cutoff,
+                              score_hint);
     }
 
 protected:
     template <typename InputIt1, typename InputIt2>
-    static ResType _distance(Range<InputIt1> s1, Range<InputIt2> s2, Args... args, ResType score_cutoff,
-                             ResType score_hint)
+    static ResType _distance(const Range<InputIt1>& s1, const Range<InputIt2>& s2, Args... args,
+                             ResType score_cutoff, ResType score_hint)
     {
         auto maximum = T::maximum(s1, s2, args...);
         ResType cutoff_similarity =
@@ -178,14 +185,25 @@ protected:
             (maximum >= score_hint) ? maximum - score_hint : static_cast<ResType>(WorstSimilarity);
         ResType sim = T::_similarity(s1, s2, std::forward<Args>(args)..., cutoff_similarity, hint_similarity);
         ResType dist = maximum - sim;
-
-        if constexpr (std::is_floating_point_v<ResType>)
-            return (dist <= score_cutoff) ? dist : 1.0;
-        else
-            return (dist <= score_cutoff) ? dist : score_cutoff + 1;
+        return _apply_distance_score_cutoff(dist, score_cutoff);
     }
 
-    SimilarityBase(){};
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : 1.0;
+    }
+
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<!std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : score_cutoff + 1;
+    }
+
+    SimilarityBase()
+    {}
     friend T;
 };
 
@@ -195,32 +213,32 @@ struct CachedNormalizedMetricBase {
     double normalized_distance(InputIt2 first2, InputIt2 last2, double score_cutoff = 1.0,
                                double score_hint = 1.0) const
     {
-        return _normalized_distance(Range(first2, last2), score_cutoff, score_hint);
+        return _normalized_distance(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
     double normalized_distance(const Sentence2& s2, double score_cutoff = 1.0, double score_hint = 1.0) const
     {
-        return _normalized_distance(Range(s2), score_cutoff, score_hint);
+        return _normalized_distance(make_range(s2), score_cutoff, score_hint);
     }
 
     template <typename InputIt2>
     double normalized_similarity(InputIt2 first2, InputIt2 last2, double score_cutoff = 0.0,
                                  double score_hint = 0.0) const
     {
-        return _normalized_similarity(Range(first2, last2), score_cutoff, score_hint);
+        return _normalized_similarity(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
     double normalized_similarity(const Sentence2& s2, double score_cutoff = 0.0,
                                  double score_hint = 0.0) const
     {
-        return _normalized_similarity(Range(s2), score_cutoff, score_hint);
+        return _normalized_similarity(make_range(s2), score_cutoff, score_hint);
     }
 
 protected:
     template <typename InputIt2>
-    double _normalized_distance(Range<InputIt2> s2, double score_cutoff, double score_hint) const
+    double _normalized_distance(const Range<InputIt2>& s2, double score_cutoff, double score_hint) const
     {
         const T& derived = static_cast<const T&>(*this);
         auto maximum = derived.maximum(s2);
@@ -234,7 +252,7 @@ protected:
     }
 
     template <typename InputIt2>
-    double _normalized_similarity(Range<InputIt2> s2, double score_cutoff, double score_hint) const
+    double _normalized_similarity(const Range<InputIt2>& s2, double score_cutoff, double score_hint) const
     {
         double cutoff_score = NormSim_to_NormDist(score_cutoff);
         double hint_score = NormSim_to_NormDist(score_hint);
@@ -243,7 +261,8 @@ protected:
         return (norm_sim >= score_cutoff) ? norm_sim : 0.0;
     }
 
-    CachedNormalizedMetricBase(){};
+    CachedNormalizedMetricBase()
+    {}
     friend T;
 };
 
@@ -255,7 +274,7 @@ struct CachedDistanceBase : public CachedNormalizedMetricBase<T> {
                      ResType score_hint = static_cast<ResType>(WorstDistance)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        return derived._distance(Range(first2, last2), score_cutoff, score_hint);
+        return derived._distance(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
@@ -263,7 +282,7 @@ struct CachedDistanceBase : public CachedNormalizedMetricBase<T> {
                      ResType score_hint = static_cast<ResType>(WorstDistance)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        return derived._distance(Range(s2), score_cutoff, score_hint);
+        return derived._distance(make_range(s2), score_cutoff, score_hint);
     }
 
     template <typename InputIt2>
@@ -271,19 +290,19 @@ struct CachedDistanceBase : public CachedNormalizedMetricBase<T> {
                        ResType score_cutoff = static_cast<ResType>(WorstSimilarity),
                        ResType score_hint = static_cast<ResType>(WorstSimilarity)) const
     {
-        return _similarity(Range(first2, last2), score_cutoff, score_hint);
+        return _similarity(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
     ResType similarity(const Sentence2& s2, ResType score_cutoff = static_cast<ResType>(WorstSimilarity),
                        ResType score_hint = static_cast<ResType>(WorstSimilarity)) const
     {
-        return _similarity(Range(s2), score_cutoff, score_hint);
+        return _similarity(make_range(s2), score_cutoff, score_hint);
     }
 
 protected:
     template <typename InputIt2>
-    ResType _similarity(Range<InputIt2> s2, ResType score_cutoff, ResType score_hint) const
+    ResType _similarity(const Range<InputIt2>& s2, ResType score_cutoff, ResType score_hint) const
     {
         const T& derived = static_cast<const T&>(*this);
         ResType maximum = derived.maximum(s2);
@@ -297,7 +316,8 @@ protected:
         return (sim >= score_cutoff) ? sim : 0;
     }
 
-    CachedDistanceBase(){};
+    CachedDistanceBase()
+    {}
     friend T;
 };
 
@@ -308,14 +328,14 @@ struct CachedSimilarityBase : public CachedNormalizedMetricBase<T> {
                      ResType score_cutoff = static_cast<ResType>(WorstDistance),
                      ResType score_hint = static_cast<ResType>(WorstDistance)) const
     {
-        return _distance(Range(first2, last2), score_cutoff, score_hint);
+        return _distance(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
     ResType distance(const Sentence2& s2, ResType score_cutoff = static_cast<ResType>(WorstDistance),
                      ResType score_hint = static_cast<ResType>(WorstDistance)) const
     {
-        return _distance(Range(s2), score_cutoff, score_hint);
+        return _distance(make_range(s2), score_cutoff, score_hint);
     }
 
     template <typename InputIt2>
@@ -324,7 +344,7 @@ struct CachedSimilarityBase : public CachedNormalizedMetricBase<T> {
                        ResType score_hint = static_cast<ResType>(WorstSimilarity)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        return derived._similarity(Range(first2, last2), score_cutoff, score_hint);
+        return derived._similarity(make_range(first2, last2), score_cutoff, score_hint);
     }
 
     template <typename Sentence2>
@@ -332,12 +352,12 @@ struct CachedSimilarityBase : public CachedNormalizedMetricBase<T> {
                        ResType score_hint = static_cast<ResType>(WorstSimilarity)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        return derived._similarity(Range(s2), score_cutoff, score_hint);
+        return derived._similarity(make_range(s2), score_cutoff, score_hint);
     }
 
 protected:
     template <typename InputIt2>
-    ResType _distance(Range<InputIt2> s2, ResType score_cutoff, ResType score_hint) const
+    ResType _distance(const Range<InputIt2>& s2, ResType score_cutoff, ResType score_hint) const
     {
         const T& derived = static_cast<const T&>(*this);
         ResType maximum = derived.maximum(s2);
@@ -345,50 +365,61 @@ protected:
         ResType hint_similarity = (maximum > score_hint) ? maximum - score_hint : 0;
         ResType sim = derived._similarity(s2, cutoff_similarity, hint_similarity);
         ResType dist = maximum - sim;
-
-        if constexpr (std::is_floating_point_v<ResType>)
-            return (dist <= score_cutoff) ? dist : 1.0;
-        else
-            return (dist <= score_cutoff) ? dist : score_cutoff + 1;
+        return _apply_distance_score_cutoff(dist, score_cutoff);
     }
 
-    CachedSimilarityBase(){};
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : 1.0;
+    }
+
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<!std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : score_cutoff + 1;
+    }
+
+    CachedSimilarityBase()
+    {}
     friend T;
 };
 
-template <typename T>
+template <typename T, typename ResType>
 struct MultiNormalizedMetricBase {
     template <typename InputIt2>
     void normalized_distance(double* scores, size_t score_count, InputIt2 first2, InputIt2 last2,
                              double score_cutoff = 1.0) const
     {
-        _normalized_distance(scores, score_count, Range(first2, last2), score_cutoff);
+        _normalized_distance(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
     void normalized_distance(double* scores, size_t score_count, const Sentence2& s2,
                              double score_cutoff = 1.0) const
     {
-        _normalized_distance(scores, score_count, Range(s2), score_cutoff);
+        _normalized_distance(scores, score_count, make_range(s2), score_cutoff);
     }
 
     template <typename InputIt2>
     void normalized_similarity(double* scores, size_t score_count, InputIt2 first2, InputIt2 last2,
                                double score_cutoff = 0.0) const
     {
-        _normalized_similarity(scores, score_count, Range(first2, last2), score_cutoff);
+        _normalized_similarity(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
     void normalized_similarity(double* scores, size_t score_count, const Sentence2& s2,
                                double score_cutoff = 0.0) const
     {
-        _normalized_similarity(scores, score_count, Range(s2), score_cutoff);
+        _normalized_similarity(scores, score_count, make_range(s2), score_cutoff);
     }
 
 protected:
     template <typename InputIt2>
-    void _normalized_distance(double* scores, size_t score_count, Range<InputIt2> s2,
+    void _normalized_distance(double* scores, size_t score_count, const Range<InputIt2>& s2,
                               double score_cutoff = 1.0) const
     {
         const T& derived = static_cast<const T&>(*this);
@@ -396,26 +427,27 @@ protected:
             throw std::invalid_argument("scores has to have >= result_count() elements");
 
         // reinterpretation only works when the types have the same size
-        int64_t* scores_i64 = nullptr;
-        if constexpr (sizeof(double) == sizeof(int64_t))
-            scores_i64 = reinterpret_cast<int64_t*>(scores);
-        else
-            scores_i64 = new int64_t[derived.result_count()];
+        ResType* scores_orig = nullptr;
 
-        Range s2_(s2);
-        derived.distance(scores_i64, derived.result_count(), s2_);
+        RAPIDFUZZ_IF_CONSTEXPR (sizeof(double) == sizeof(ResType))
+            scores_orig = reinterpret_cast<ResType*>(scores);
+        else
+            scores_orig = new ResType[derived.result_count()];
+
+        derived.distance(scores_orig, derived.result_count(), s2);
 
         for (size_t i = 0; i < derived.get_input_count(); ++i) {
             auto maximum = derived.maximum(i, s2);
-            double norm_dist = static_cast<double>(scores_i64[i]) / static_cast<double>(maximum);
+            double norm_dist =
+                (maximum != 0) ? static_cast<double>(scores_orig[i]) / static_cast<double>(maximum) : 0.0;
             scores[i] = (norm_dist <= score_cutoff) ? norm_dist : 1.0;
         }
 
-        if constexpr (sizeof(double) != sizeof(int64_t)) delete[] scores_i64;
+        RAPIDFUZZ_IF_CONSTEXPR (sizeof(double) != sizeof(ResType)) delete[] scores_orig;
     }
 
     template <typename InputIt2>
-    void _normalized_similarity(double* scores, size_t score_count, Range<InputIt2> s2,
+    void _normalized_similarity(double* scores, size_t score_count, const Range<InputIt2>& s2,
                                 double score_cutoff) const
     {
         const T& derived = static_cast<const T&>(*this);
@@ -427,18 +459,19 @@ protected:
         }
     }
 
-    MultiNormalizedMetricBase(){};
+    MultiNormalizedMetricBase()
+    {}
     friend T;
 };
 
 template <typename T, typename ResType, int64_t WorstSimilarity, int64_t WorstDistance>
-struct MultiDistanceBase : public MultiNormalizedMetricBase<T> {
+struct MultiDistanceBase : public MultiNormalizedMetricBase<T, ResType> {
     template <typename InputIt2>
     void distance(ResType* scores, size_t score_count, InputIt2 first2, InputIt2 last2,
                   ResType score_cutoff = static_cast<ResType>(WorstDistance)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        derived._distance(scores, score_count, Range(first2, last2), score_cutoff);
+        derived._distance(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
@@ -446,26 +479,27 @@ struct MultiDistanceBase : public MultiNormalizedMetricBase<T> {
                   ResType score_cutoff = static_cast<ResType>(WorstDistance)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        derived._distance(scores, score_count, Range(s2), score_cutoff);
+        derived._distance(scores, score_count, make_range(s2), score_cutoff);
     }
 
     template <typename InputIt2>
     void similarity(ResType* scores, size_t score_count, InputIt2 first2, InputIt2 last2,
                     ResType score_cutoff = static_cast<ResType>(WorstSimilarity)) const
     {
-        _similarity(scores, score_count, Range(first2, last2), score_cutoff);
+        _similarity(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
     void similarity(ResType* scores, size_t score_count, const Sentence2& s2,
                     ResType score_cutoff = static_cast<ResType>(WorstSimilarity)) const
     {
-        _similarity(scores, score_count, Range(s2), score_cutoff);
+        _similarity(scores, score_count, make_range(s2), score_cutoff);
     }
 
 protected:
     template <typename InputIt2>
-    void _similarity(ResType* scores, size_t score_count, Range<InputIt2> s2, ResType score_cutoff) const
+    void _similarity(ResType* scores, size_t score_count, const Range<InputIt2>& s2,
+                     ResType score_cutoff) const
     {
         const T& derived = static_cast<const T&>(*this);
         derived._distance(scores, score_count, s2);
@@ -477,24 +511,25 @@ protected:
         }
     }
 
-    MultiDistanceBase(){};
+    MultiDistanceBase()
+    {}
     friend T;
 };
 
 template <typename T, typename ResType, int64_t WorstSimilarity, int64_t WorstDistance>
-struct MultiSimilarityBase : public MultiNormalizedMetricBase<T> {
+struct MultiSimilarityBase : public MultiNormalizedMetricBase<T, ResType> {
     template <typename InputIt2>
     void distance(ResType* scores, size_t score_count, InputIt2 first2, InputIt2 last2,
                   ResType score_cutoff = static_cast<ResType>(WorstDistance)) const
     {
-        _distance(scores, score_count, Range(first2, last2), score_cutoff);
+        _distance(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
     void distance(ResType* scores, size_t score_count, const Sentence2& s2,
-                  ResType score_cutoff = WorstDistance) const
+                  ResType score_cutoff = static_cast<ResType>(WorstDistance)) const
     {
-        _distance(scores, score_count, Range(s2), score_cutoff);
+        _distance(scores, score_count, make_range(s2), score_cutoff);
     }
 
     template <typename InputIt2>
@@ -502,7 +537,7 @@ struct MultiSimilarityBase : public MultiNormalizedMetricBase<T> {
                     ResType score_cutoff = static_cast<ResType>(WorstSimilarity)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        derived._similarity(scores, score_count, Range(first2, last2), score_cutoff);
+        derived._similarity(scores, score_count, make_range(first2, last2), score_cutoff);
     }
 
     template <typename Sentence2>
@@ -510,12 +545,12 @@ struct MultiSimilarityBase : public MultiNormalizedMetricBase<T> {
                     ResType score_cutoff = static_cast<ResType>(WorstSimilarity)) const
     {
         const T& derived = static_cast<const T&>(*this);
-        derived._similarity(scores, score_count, Range(s2), score_cutoff);
+        derived._similarity(scores, score_count, make_range(s2), score_cutoff);
     }
 
 protected:
     template <typename InputIt2>
-    void _distance(ResType* scores, size_t score_count, Range<InputIt2> s2, ResType score_cutoff) const
+    void _distance(ResType* scores, size_t score_count, const Range<InputIt2>& s2, ResType score_cutoff) const
     {
         const T& derived = static_cast<const T&>(*this);
         derived._similarity(scores, score_count, s2);
@@ -523,16 +558,28 @@ protected:
         for (size_t i = 0; i < derived.get_input_count(); ++i) {
             ResType maximum = derived.maximum(i, s2);
             ResType dist = maximum - scores[i];
-
-            if constexpr (std::is_floating_point_v<ResType>)
-                scores[i] = (dist <= score_cutoff) ? dist : 1.0;
-            else
-                scores[i] = (dist <= score_cutoff) ? dist : score_cutoff + 1;
+            scores[i] = _apply_distance_score_cutoff(dist, score_cutoff);
         }
     }
 
-    MultiSimilarityBase(){};
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : 1.0;
+    }
+
+    template <typename U>
+    static rapidfuzz::rf_enable_if_t<!std::is_floating_point<U>::value, U>
+    _apply_distance_score_cutoff(U score, U score_cutoff)
+    {
+        return (score <= score_cutoff) ? score : score_cutoff + 1;
+    }
+
+    MultiSimilarityBase()
+    {}
     friend T;
 };
 
-} // namespace rapidfuzz::detail
+} // namespace detail
+} // namespace rapidfuzz
